﻿using System;
using System.Windows;

namespace Hardcodet.Wpf.Util
{
  /// <summary>
  /// Provides convenience methods to display message boxes. In order to set
  /// the title of the message box, provide the class with a resolver method
  /// through <see cref="SetDialogTitleQuery"/>.
  /// </summary>
  public static class Dialogs
  {

    #region dialog title

    /// <summary>
    /// Resolves the application's name at runtime. Returns an empty
    /// string by default.
    /// </summary>
    private static Func<string> TitleQuery = () => String.Empty;


    /// <summary>
    /// Sets a function that resolves the application's name at runtime.
    /// </summary>
    /// <param name="query">A function that can be used to resolve the
    /// dialog title at runtime.</param>
    /// <exception cref="ArgumentNullException">If <paramref name="query"/>
    /// is a null reference.</exception>
    public static void SetDialogTitleQuery(Func<string> query)
    {
      if (query == null) throw new ArgumentNullException("query");
      TitleQuery = query;
    }


    /// <summary>
    /// Gets the title that is used when displaying a dialog on the UI.
    /// This property returns an empty string if no query function
    /// was assigned through <see cref="SetDialogTitleQuery"/>.
    /// </summary>
    public static string DialogTitle
    {
      get { return TitleQuery(); }
    }

    #endregion


    /// <summary>
    /// Displays an error dialog with a given message.
    /// </summary>
    /// <param name="message">The message to be displayed.</param>
    public static void ShowError(string message)
    {
      ShowMessage(message, MessageBoxImage.Error);
    }



    /// <summary>
    /// Displays an error dialog with a given message.
    /// </summary>
    /// <param name="message">The message to be displayed.</param>
    public static void ShowInformation(string message)
    {
      ShowMessage(message, MessageBoxImage.Information);
    }



    /// <summary>
    /// Displays an error dialog with a given message.
    /// </summary>
    /// <param name="message">The message to be displayed.</param>
    public static void ShowWarning(string message)
    {
      ShowMessage(message, MessageBoxImage.Warning);
    }



    /// <summary>
    /// Displays an error dialog with a given message and icon.
    /// </summary>
    /// <param name="message">The message to be displayed.</param>
    /// <param name="icon">The icon to be displayed with the message.</param>
    public static void ShowMessage(string message, MessageBoxImage icon)
    {
      MessageBox.Show(message, DialogTitle, MessageBoxButton.OK, icon);
    }



    /// <summary>
    /// Displays an OK / Cancel dialog and returns the user input.
    /// </summary>
    /// <param name="message">The message to be displayed.</param>
    /// <param name="icon">The icon to be displayed.</param>
    /// <returns>User selection.</returns>
    public static MessageBoxResult ShowOkCancel(string message, MessageBoxImage icon)
    {
      return MessageBox.Show(message, DialogTitle, MessageBoxButton.OKCancel, icon);
    }



    /// <summary>
    /// Displays a Yes/No dialog and returns the user input.
    /// </summary>
    /// <param name="message">The message to be displayed.</param>
    /// <param name="icon">The icon to be displayed.</param>
    /// <returns>User selection.</returns>
    public static MessageBoxResult ShowYesNo(string message, MessageBoxImage icon)
    {
      return MessageBox.Show(message, DialogTitle, MessageBoxButton.YesNo, icon);
    }


    /// <summary>
    /// Displays an Yes / No / Cancel dialog and returns the user input.
    /// </summary>
    /// <param name="message">The message to be displayed.</param>
    /// <param name="icon">The icon to be displayed.</param>
    /// <returns>User selection.</returns>
    public static MessageBoxResult ShowYesNoCancel(string message, MessageBoxImage icon)
    {
      return MessageBox.Show(message, DialogTitle, MessageBoxButton.YesNoCancel, icon);
    }
  }
}